@@ -5,7 +5,7 @@ use base qw/DBIx::Class/;
 use strict;
 use warnings;
 
-our $VERSION = '0.02_01';
+our $VERSION = '0.900001_02';
 
 ## On create/insert, add new entry to AuditLog
 
@@ -18,24 +18,34 @@ our $VERSION = '0.02_01';
 #     $class->next::method($attrs, @rest);
 # }
 
+sub _journal_schema {
+    my $self = shift;
+    $self->result_source->schema->_journal_schema;
+}
+
 sub insert
+{
+    my ($self, @args) = @_;
+
+    return if($self->in_storage);
+
+    my $res = $self->next::method(@args);
+
+    $self->journal_log_insert();
+
+    return $res;
+}
+
+sub journal_log_insert
 {
     my ($self) = @_;
 
-    return if($self->in_storage);
-    ## create new transaction here?
-    my $res = $self->next::method();
-    if($self->in_storage)
-    {
-        my $s_name = $self->result_source->source_name();
-        my $al = $self->result_source->schema->_journal_schema->resultset("${s_name}AuditLog");
-        $al->update_or_create({
-            ( map { $_ => $self->get_column($_)} $self->primary_columns ),
-            created => { changeset_id => $al->result_source->schema->current_changeset },
-        });
+    if ( $self->in_storage ) {
+        my $j = $self->_journal_schema;
+        my $change_id = $j->journal_create_change()->id;
+        $j->journal_update_or_create_log_entry( $self, create_id => $change_id );
+        $j->journal_record_in_history( $self, audit_change_id => $change_id );
     }
-
-    return $res;
 }
 
 ## On delete, update delete_id of AuditLog
@@ -44,40 +54,40 @@ sub delete
 {
     my ($self, @rest) = @_;
     $self->next::method(@rest);
+    $self->journal_log_delete(@rest);
+}
 
-    if(!$self->in_storage)
-    {
-        my $s_name = $self->result_source->source_name();
-        my $al = $self->result_source->schema->_journal_schema->resultset("${s_name}AuditLog");
-        my $alentry = $al->find_or_create({ map { $_ => $self->get_column($_)} $self->primary_columns });
-         
-        ## bulk_update doesnt do "create new item on update of rel-accessor with hashref, yet
-        my $change = $self->result_source->schema->_journal_schema->resultset('ChangeLog')->create({ changeset_id => $self->result_source->schema->_journal_schema->current_changeset });
-        $alentry->delete_id($change->id);
-        $alentry->update();
+sub journal_log_delete
+{
+    my ($self) = @_;
+
+    unless ($self->in_storage) {
+        my $j = $self->_journal_schema;
+        $j->journal_update_or_create_log_entry( $self, delete_id => $j->journal_create_change->id );
     }
-    
 }
 
 ## On update, copy previous row's contents to AuditHistory
 
 sub update 
+{
+    my ($self, $upd, @rest) = @_;
+    $self->journal_log_update($upd, @rest);
+    $self->next::method($upd, @rest);
+}
+
+sub journal_log_update 
 {
     my ($self, $upd, @rest) = @_;
 
     if($self->in_storage)
     {
-        my $s_name = $self->result_source->source_name();
-        my $ah = $self->result_source->schema->_journal_schema->resultset("${s_name}AuditHistory");
+        my $j = $self->_journal_schema;
 
-        my $obj = $self->result_source->resultset->find( $self->ident_condition );
-        $ah->create({
-            $obj->get_columns,
-            change => { changeset_id => $ah->result_source->schema->current_changeset },
-        });
+        my $change = $j->journal_create_change;
+        my $prev = $self->result_source->resultset->find( $self->ident_condition );
+        $j->journal_record_in_history( $prev, audit_change_id => $change );
     }
-
-    $self->next::method($upd, @rest);
 }
 
 =head1 NAME
